require 'pathname'

class Integer 
  def self.scan_from_svipack buf 
    [ buf[0..3].unpack('L')[0], buf[4..-1] ]
  end

  def self.put_to_svipack val
    [val].pack('L')
  end
end

class String
  def self.scan_from_svipack buf 
    len, buf = Integer.scan_from_svipack buf

    [ buf[0..len-1], buf[len..-1] ]
  end

  def self.put_to_svipack val
    res = ''
    res << Integer.put_to_svipack(val.length)
    res << val
  end
end

module SelfString
  @len

  def scan_from_svipack buf
    raise UndefinedError, self unless @len

    [ buf[0..@len-1], buf[@len..-1] ]
  end

  def len= length
    raise ArgumentError unless length.kind_of? Integer

    @len = length
  end
end

class LUNDesc < Array
  def self.scan_from_svipack buf 
    count, buf = Integer.scan_from_svipack buf

    luns = []

    # TODO: Kill Constant «8»
    raise ArgumentError, "LUNDesc: error count #{count}" unless count <= 8 and count > 0

    count.times do
      start, buf = Integer.scan_from_svipack buf
      len, buf   = Integer.scan_from_svipack buf
      mode, buf  = Integer.scan_from_svipack buf
      flags, buf = Integer.scan_from_svipack buf

      luns << start << len << mode << flags
    end

    [ luns, buf ]
  end

  def self.put_to_svipack val
    raise "LUNDesc elements array.count%4 shall be 0" unless val.count%4 == 0
    res = ''

    res << Integer.put_to_svipack(val.count/4)

    val.each do |elem|
      res << Integer.put_to_svipack(elem)
    end

    res
  end
end

module SVISubsystemNVM
  def nvm_load_file bank, offset, path
    raise ArgumentError, 'file (%s) is not readable' % path.to_s unless Pathname.new(path).readable_real?

    data = IO.binread(path).unpack 'C*'
    data = data + [0]*(data.length % 1024 > 0 ? (1024 - data.length%1024) : 0)
    data = data.each_slice(1024).to_a

    data.each_with_index do |packet, i|
      nvm_write bank, offset + i*1024, packet.pack('C*')
    end

    [0]
  end

  def nvm_verify_file bank, offset, path
    raise ArgumentError, 'file (%s) is not readable' % path.to_s unless Pathname.new(path).readable_real?

    data = IO.binread(path).unpack 'C*'
    data = data + [0]*(data.length % 1024 > 0 ? (1024 - data.length%1024) : 0)
    data = data.each_slice(1024).to_a

    data.each_with_index do |packet, i|
      res = nvm_read(bank, offset + i*1024, 1024)[0]

      return [1, res, packet.pack('C*')] if res != packet.pack('C*')
    end

    [0]
  end
end

# vim: sw=2 sts=2 ts=8:

