require 'libusb'

require 'svi/api'
require 'svi/api.generated'


class USB < LIBUSB::Context
  @vidpids

  def initialize(vid, pid)
    @vidpids = [
      {:idVendor => vid, :idProduct => pid}
    ]

    super()
  end

  def svi_devices
    devs = []
    @vidpids.each { |vidpid|
      devs = devs + devices(vidpid)
    }

    devs.each { |dev| dev.extend SVIDevice  }

    devs
  end
end

module SVICall
  SVI_PACKET_SIZE = 2048

  def call(*_args)
    args = _args.reverse
    noreturn = false
    resp = nil
    packet = ""

    raise ArgumentError, 'Wrong arguments count' if args.count == 0

    if args[-1] == :noreturn
      noreturn = true
      args.pop
    end

    name = args.pop

    raise ArgumentError, "Invalid syscall name #{name}" unless $svi_api_desc.has_key? name
    raise ArgumentError, 'SVI is not configured' unless svi_configured?

    packet << name.to_s << "\0"

    args.reverse!

    $svi_api_desc[name][:args].each_with_index do |arg, i|
      packet << arg.put_to_svipack(args[i])
    end

    raise ArgumentError, 'Too long svc call' if packet.length > SVI_PACKET_SIZE

    packet << ([0]*(SVI_PACKET_SIZE-packet.length)).pack('C*')

    res = nil

    open_interface(svi_if) do |handle|
      res = handle.bulk_transfer(:endpoint => svi_if_ep_out, :dataOut => packet)

      raise IOError, 'Sending failed' unless res == SVI_PACKET_SIZE

      resp = []

      res = handle.bulk_transfer(:endpoint => svi_if_ep_in | 0x80, :dataIn => SVI_PACKET_SIZE, :timeout => @svi_timeout) unless $svi_api_desc[name][:response].empty? || noreturn
    end

    unless $svi_api_desc[name][:response].empty? || noreturn
      $svi_api_desc[name][:response].each do |resptype|
        resp_token, res = resptype.scan_from_svipack res
        resp << resp_token
      end
    else
      resp << 0
    end

    resp
  end

  def method_missing *args
    call *args
  end
end

module SBITxRx
  attr_accessor :sbi_if
  attr_accessor :sbi_if_ep_out
  attr_accessor :sbi_if_ep_in
  attr_accessor :sbi_timeout

  def sbi_configured?
    !(@sbi_if.nil? or @sbi_if_ep_out.nil? or @sbi_if_ep_in.nil?)
  end

  def sbi_send data
    raise ArgumentError unless sbi_configured? 

    open_interface(sbi_if) { |handle|
      res = handle.bulk_transfer(:endpoint => sbi_if_ep_out, :dataOut => data)
    }
  end

  def sbi_receive length
    raise ArgumentError if @sbi_if.nil? or @sbi_if_ep_out.nil? or @sbi_if_ep_in.nil?

    open_interface(sbi_if) { |handle|
      res = handle.bulk_transfer(:endpoint => sbi_if_ep_in | 0x80, :dataIn => length, :timeout => @sbi_timeout) 
    }
  end
end

module SVIDevice
  attr_accessor :svi_if
  attr_accessor :svi_if_ep_out
  attr_accessor :svi_if_ep_in
  attr_accessor :svi_timeout

  include SVICall
  include SBITxRx
  include SVISubsystemNVM

  def svi_configured?
    !(@svi_if.nil? or @svi_if_ep_out.nil? or @svi_if_ep_in.nil? or @svi_timeout.nil?)
  end
end

# vim: sw=2 sts=2 ts=8:

